@@ -5,6 +5,8 @@ use warnings;
 use parent 'Exporter';
 use Nephia::Request;
 use Nephia::Response;
+use Nephia::GlobalVars;
+use Nephia::Context;
 use Plack::Builder;
 use Router::Simple;
 use Nephia::View;
@@ -16,46 +18,51 @@ use Scalar::Util qw/blessed/;
 use Module::Load ();
 
 our @EXPORT = qw[ get post put del path req res param path_param nip run config app nephia_plugins base_dir cookie set_cookie ];
-our $MAPPER = Router::Simple->new;
-our $VIEW;
-our $CONFIG = {};
-our $CHARSET = Encode::find_encoding('UTF-8');
-our $APP_MAP = {};
-our $APP_CODE = {};
-our $APP_ROOT;
-our $COOKIE;
+our $CONTEXT;
+
+Nephia::GlobalVars->set(
+    mapper   => Router::Simple->new,
+    view     => undef,
+    config   => {},
+    charset  => Encode::find_encoding('UTF-8'),
+    app_map  => {},
+    app_code => {},
+    app_root => undef,
+    json     => JSON->new->utf8,
+);
 
 sub _path {
     my ( $path, $code, $methods, $target_class ) = @_;
     my $caller = caller();
+    my ($app_map, $app_code, $mapper) = Nephia::GlobalVars->get(qw/app_map app_code mapper/);
 
     if (
         $target_class
-        && exists $APP_MAP->{$target_class}
-        && $APP_MAP->{$target_class}->{path}
+        && exists $app_map->{$target_class}
+        && $app_map->{$target_class}{path}
     ) {
         # setup for submapping one more
-        $APP_CODE->{$target_class} ||= {};
-        if (!exists $APP_CODE->{$target_class}->{$path}) {
-            $APP_CODE->{$target_class}->{$path} = {
+        $app_code->{$target_class} ||= {};
+        if (!exists $app_code->{$target_class}{$path}) {
+            $app_code->{$target_class}{$path} = {
                 code => $code,
                 methods => $methods,
             };
         }
 
         $path =~ s!^/!!g;
-        my @paths = ($APP_MAP->{$target_class}->{path});
+        my @paths = ($app_map->{$target_class}->{path});
         push @paths, $path if length($path) > 0;
         $path = join '/', @paths;
     }
 
-    $MAPPER->connect(
+    $mapper->connect(
         $path,
         {
             action => sub {
-                my $req = Nephia::Request->new( shift );
-                $req->{path_param} = shift;
-                local $COOKIE = $req->cookies;
+                my ($env, $path_param) = @_;
+                local $CONTEXT = Nephia::Context->new;
+                my $req = _process_request($env, $path_param);
                 no strict qw[ refs subs ];
                 no warnings qw[ redefine ];
                 local *{$caller."::req"} = sub{ $req };
@@ -66,27 +73,7 @@ sub _path {
                 local *{$caller."::path_param"} = sub (;$) { $req->path_param(shift) };
                 local *{$caller."::nip"} = sub (;$) { $req->nip(shift) };
                 my $res = $code->( $req, $req->path_param );
-                my $rtn;
-                if ( ref $res eq 'HASH' ) {
-                    $rtn = eval { $res->{template} } ?
-                        render( $res ) :
-                        json_res( $res )
-                    ;
-                }
-                elsif ( blessed $res && $res->isa('Plack::Response') ) {
-                    $rtn = $res->finalize;
-                }
-                else {
-                    $rtn = $res;
-                }
-                if ($COOKIE) {
-                    my $res_obj = Nephia::Response->new(@$rtn);
-                    for my $key (keys %$COOKIE) {
-                        $res_obj->cookies->{$key} = $COOKIE->{$key};
-                    }
-                    $rtn = $res_obj->finalize;
-                }
-                return $rtn;
+                return _process_response($res);
             },
         },
         $methods ? { method => $methods } : undef,
@@ -94,22 +81,79 @@ sub _path {
 
 }
 
+sub _process_request {
+    my ($raw_env, $path_param) = @_;
+    my $env = process_env($raw_env);
+    my $req = Nephia::Request->new($env);
+    $req->{path_param} = $path_param;
+    $CONTEXT->{cookie} = $req->cookies;
+    return $req;
+}
+
+sub process_env {
+    my $env = shift;
+    return $env;
+}
+
+sub _process_response {
+    my $raw_res = shift;
+    my $res;
+    if ( ref $raw_res eq 'HASH' ) {
+        $res = Nephia::Response->new(@{
+            eval { $raw_res->{template} } ? 
+                render($raw_res) : 
+                json_res($raw_res)
+        });
+    }
+    elsif ( blessed $raw_res && $raw_res->isa('Plack::Response') ) {
+        $res = $raw_res;
+    }
+    else {
+        $res = Nephia::Response->new(@{$raw_res});
+    }
+    if ($CONTEXT->{cookie}) {
+        for my $key (keys %{$CONTEXT->{cookie}}) {
+            $res->cookies->{$key} = $CONTEXT->{cookie}{$key};
+        }
+    }
+    $res = process_response($res);
+    if (ref($res->body) eq 'ARRAY') {
+        $res->body->[0] = process_content($res->body->[0]);
+    }
+    else {
+        $res->body( process_content($res->body) );
+    }
+    return $res->finalize;
+}
+
+sub process_response {
+    my $res = shift;
+    return $res;
+}
+
+sub process_content {
+    my $content = shift;
+    return $content;
+}
+
 sub _submap {
     my ( $path, $package, $base_class ) = @_;
 
+    my ($app_map, $app_code) = Nephia::GlobalVars->get(qw/app_map app_code/);
+
     if (!($package =~ s/^\+//g)) {
         $package = join '::', $base_class, $package;
     }
 
-    $APP_MAP->{$package}->{path} = $path;
-    if (!$APP_CODE->{$package}) {
+    $app_map->{$package}{path} = $path;
+    if (!$app_code->{$package}) {
         Module::Load::load($package);
         $package->import if $package->can('import');
     }
     else {
-         for my $suffix_path (keys %{$APP_CODE->{$package}}) {
-            my $app_code = $APP_CODE->{$package}->{$suffix_path};
-            _path ($suffix_path, $app_code->{code}, $app_code->{methods}, $package);
+         for my $suffix_path (keys %{$app_code->{$package}}) {
+            my $sub_app_code = $app_code->{$package}->{$suffix_path};
+            _path ($suffix_path, $sub_app_code->{code}, $sub_app_code->{methods}, $package);
         }
     }
 }
@@ -177,8 +221,10 @@ sub res (&) {
 sub run {
     my $class = shift;
     my $base_dir = base_dir($class);
-    $CONFIG = scalar @_ > 1 ? +{ @_ } : $_[0];
-    $VIEW = Nephia::View->new( ( $CONFIG->{view} ? %{$CONFIG->{view}} : () ), template_path => File::Spec->catdir($base_dir, 'view') );
+    my $config = scalar @_ > 1 ? +{ @_ } : $_[0];
+    my $view = Nephia::View->new(($config->{view} ? %{$config->{view}} : ()), template_path => File::Spec->catdir($base_dir, 'view'));
+
+    Nephia::GlobalVars->set(config => $config, view => $view);
 
     my $root = File::Spec->catfile($base_dir, 'root');
     return builder {
@@ -189,9 +235,10 @@ sub run {
 
 sub app {
     my $class = shift;
+    my $mapper = Nephia::GlobalVars->get('mapper');
     return sub {
         my $env = shift;
-        if ( my $p = $MAPPER->match($env) ) {
+        if ( my $p = $mapper->match($env) ) {
             $p->{action}->($env, $p);
         }
         else {
@@ -200,13 +247,10 @@ sub app {
     };
 }
 
-{
-    my $_json;
-    sub _json { $_json ||= JSON->new->utf8 }
-}
 sub json_res {
-    my $res = shift;
-    my $body = _json->encode( $res );
+    my $res  = shift;
+    my $json = Nephia::GlobalVars->get('json');
+    my $body = $json->encode( $res );
     return [ 200,
         [
             'Content-type'           => 'application/json',
@@ -219,9 +263,10 @@ sub json_res {
 }
 
 sub render {
-    my $res = shift;
-    my $charset = delete $res->{charset} || $CHARSET;
-    my $body = $VIEW->render( $res->{template}, $res );
+    my $res     = shift;
+    my $charset = delete $res->{charset} || Nephia::GlobalVars->get('charset');
+    my $view    = Nephia::GlobalVars->get('view');
+    my $body    = $view->render( $res->{template}, $res );
     return [ 200,
         [ 'Content-type' => "text/html; charset=$charset" ],
         [ Encode::encode( $charset, $body ) ]
@@ -229,14 +274,12 @@ sub render {
 }
 
 sub config (@) {
-    if ( scalar @_ > 0 ) {
-        $CONFIG =
-            scalar @_ > 1 ? { @_ } :
-            ref $_[0] eq 'HASH' ? $_[0] :
-            do( $_[0] )
-        ;
-    }
-    return $CONFIG;
+    Nephia::GlobalVars->set(config => 
+        scalar @_ > 1 ? { @_ } :
+        ref $_[0] eq 'HASH' ? $_[0] :
+        do( $_[0] )
+    ) if scalar(@_) > 0;
+    return Nephia::GlobalVars->get('config');
 };
 
 sub nephia_plugins (@) {
@@ -262,15 +305,32 @@ sub _export_plugin_functions {
     my ($plugin, $pkg, $opt) = @_;
 
     Module::Load::load($plugin);
-
-    $plugin->import if $plugin->can('import');
-    $plugin->load($pkg, $opt) if $plugin->can('load');
     {
         no strict 'refs';
         no warnings qw/redefine prototype/;
+        *{$plugin.'::context'} = sub { 
+            my ($key, $val) = @_;
+            $CONTEXT->{$key} = $val if defined $key && defined $val;
+            return defined $key ? $CONTEXT->{$key} : $CONTEXT;
+        };
+
+        $plugin->import if $plugin->can('import');
+        $plugin->load($pkg, $opt) if $plugin->can('load');
+
         for my $func ( @{"${plugin}::EXPORT"} ){
             *{"$pkg\::$func"} = $plugin->can($func);
         }
+        for my $func (qw/process_env process_response process_content/) {
+            my $plugin_func = $plugin->can($func);
+            if ($plugin_func) {
+                my $orig = \&{$func};
+                *$func = sub {
+                    my $in = shift;
+                    my $out = $plugin_func->($orig->($in));
+                    return $out;
+                }; 
+            }
+        }
     }
 }
 
@@ -297,12 +357,12 @@ sub base_dir {
 
 sub set_cookie ($$){
     my ($key, $val) = @_;
-    $COOKIE->{$key} = $val;
+    $CONTEXT->{cookie}{$key} = $val;
 }
 
 sub cookie ($) {
     my $key = shift;
-    $COOKIE->{$key};
+    $CONTEXT->{cookie}{$key};
 }
 
 1;
